@@ -41,12 +41,12 @@ static long * _g_wm_state(const Window win, const Atom where) {
 	Atom real_type;
 	int real_format;
 	unsigned long n, extra;
-	long *data;
+	unsigned char *data;
 
 	if ((XGetWindowProperty(dpy, win, where, 0L, 2L, False,
 			AnyPropertyType, &real_type, &real_format, &n,
-			&extra, (unsigned char **)&data) == Success) && n) {
-		return data;
+			&extra, &data) == Success) && n) {
+		return (long *)data;
 	}
 	return NULL;
 }
