@@ -18,6 +18,8 @@ void handle_key_event(XKeyEvent *e) {
 		c = current;
 	if (c) {
 		switch (key) {
+		/* Sorry about all these if (0)s, but they actually
+		 * *do* do something useful... */
 		case KEY_LEFT:
 			c->x -= 16; if (0)
 		case KEY_DOWN:
@@ -29,11 +31,11 @@ void handle_key_event(XKeyEvent *e) {
 		case KEY_TOPLEFT:
 			{ c->x = c->border; c->y = c->border; } if (0)
 		case KEY_TOPRIGHT:
-			{ c->x = DisplayWidth(dpy, screen)-c->width-c->border; c->y = c->border; } if (0)
+			{ c->x = DisplayWidth(dpy, c->screen->screen)-c->width-c->border; c->y = c->border; } if (0)
 		case KEY_BOTTOMLEFT:
-			{ c->x = c->border; c->y = DisplayHeight(dpy, screen)-c->height-c->border; } if (0)
+			{ c->x = c->border; c->y = DisplayHeight(dpy, c->screen->screen)-c->height-c->border; } if (0)
 		case KEY_BOTTOMRIGHT:
-			{ c->x = DisplayWidth(dpy, screen)-c->width-c->border; c->y = DisplayHeight(dpy, screen)-c->height-c->border; }
+			{ c->x = DisplayWidth(dpy, c->screen->screen)-c->width-c->border; c->y = DisplayHeight(dpy, c->screen->screen)-c->height-c->border; }
 			move(c, 1);
 			setmouse(c->window, c->width + c->border - 1,
 					c->height + c->border - 1);
@@ -56,19 +58,20 @@ void handle_key_event(XKeyEvent *e) {
 			break;
 #ifdef VWM
 		case KEY_FIX:
-			XSetWindowBackground(dpy, c->parent,
-					RD_DESK(c) == STICKY ? fg.pixel : fc.pixel);
-			XClearWindow(dpy, c->parent);
-			WR_DESK(c, ( RD_DESK(c) == STICKY ? GET_VDESK : STICKY ) );
+			c->vdesk = c->vdesk == STICKY ? vdesk : STICKY;
+#ifdef VDESK_BOTH
+			spawn_vdesk(c->vdesk, c);
+#endif
+			client_update_current(c, current);
 			break;
 #endif
 		}
 	}
 	switch(key) {
 		case KEY_NEW:
-			spawn(opt_term ? opt_term : DEF_TERM); break;
+			spawn(opt_term); break;
 		case KEY_NEXT:
-			next(c);
+			next();
 			break;
 #ifdef VWM
 		case XK_1: case XK_2: case XK_3: case XK_4:
@@ -76,12 +79,12 @@ void handle_key_event(XKeyEvent *e) {
 			switch_vdesk(KEY_TO_VDESK(key));
 			break;
 		case KEY_PREVDESK:
-			if (GET_VDESK > KEY_TO_VDESK(XK_1))
-				switch_vdesk(GET_VDESK - 1);
+			if (vdesk > KEY_TO_VDESK(XK_1))
+				switch_vdesk(vdesk - 1);
 			break;
 		case KEY_NEXTDESK:
-			if (GET_VDESK < KEY_TO_VDESK(XK_8))
-				switch_vdesk(GET_VDESK + 1);
+			if (vdesk < KEY_TO_VDESK(XK_8))
+				switch_vdesk(vdesk + 1);
 			break;
 #endif
 	}
@@ -91,7 +94,7 @@ void handle_key_event(XKeyEvent *e) {
 void handle_button_event(XButtonEvent *e) {
 	Client *c = find_client(e->window);
 
-	if (c && e->window != root) {
+	if (c && e->window != c->screen->root) {
 		switch (e->button) {
 			case Button1:
 				move(c, 0); break;
@@ -114,15 +117,13 @@ void handle_configure_request(XConfigureRequestEvent *e) {
 		ungravitate(c);
 		if (e->value_mask & CWWidth) c->width = e->width;
 		if (e->value_mask & CWHeight) c->height = e->height;
-		if (e->value_mask & CWX) {
-			c->x = e->x;
-			if (c->x == 0 && c->width >= DisplayWidth(dpy, screen))
-				c->x -= c->border;
+		if (e->value_mask & CWX) c->x = e->x;
+		if (e->value_mask & CWY) c->y = e->y;
+		if (c->x == 0 && c->width >= DisplayWidth(dpy, c->screen->screen)) {
+			c->x -= c->border;
 		}
-		if (e->value_mask & CWY) {
-			c->y = e->y;
-			if (c->y == 0 && c->height >= DisplayHeight(dpy, screen))
-				c->y -= c->border;
+		if (c->y == 0 && c->height >= DisplayHeight(dpy, c->screen->screen)) {
+			c->y -= c->border;
 		}
 		gravitate(c);
 
@@ -133,6 +134,9 @@ void handle_configure_request(XConfigureRequestEvent *e) {
 		wc.border_width = 0;
 		XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
 		send_config(c);
+#ifdef DEBUG
+		fprintf(stderr, "handle_configure_request() : window configured to %dx%d+%d+%d\n", wc.width, wc.height, wc.x, wc.y);
+#endif
 	}
 
 	wc.x = c ? c->border : e->x;
@@ -147,15 +151,17 @@ void handle_map_request(XMapRequestEvent *e) {
 
 	if (c) {
 #ifdef VWM
-		if (RD_DESK(c) != GET_VDESK)
-			switch_vdesk(RD_DESK(c));
+		if (c->vdesk != vdesk)
+			switch_vdesk(c->vdesk);
 #endif
 		unhide(c, RAISE);
 	} else {
+		XWindowAttributes attr;
 #ifdef DEBUG
-		fprintf(stderr, "event:map->make_new_client(); ");
+		fprintf(stderr, "handle_map_request() : don't know this window, calling make_new_client();\n");
 #endif
-		make_new_client(e->window);
+		XGetWindowAttributes(dpy, e->window, &attr);
+		make_new_client(e->window, find_screen(attr.root));
 	}
 }
 
@@ -164,7 +170,8 @@ void handle_unmap_event(XUnmapEvent *e) {
 
 	if (c) {
 #ifdef DEBUG
-		fprintf(stderr, "EVENT: unmap, ignore_unmap = %d\n", c->ignore_unmap);
+		/* fprintf(stderr, "handle_unmap_event() : ignore_unmap = %d\n", c->ignore_unmap);
+		 * */
 #endif
 		if (c->ignore_unmap) c->ignore_unmap--;
 		else remove_client(c);
@@ -198,54 +205,44 @@ void handle_property_change(XPropertyEvent *e) {
 	if (c) {
 		if (e->atom == XA_WM_NORMAL_HINTS)
 				XGetWMNormalHints(dpy, c->window, c->size, &dummy);
-#ifdef VDESK_BOTH
-		/* Vdesk-hack. Evilwm does not use iconifying, so whenever
-		* vdesk tries to set it's "iconifyed" flag we remove it again.
-		* Vdesk assumes that if a window is not visible, but marked
-		* as belonging to the current window, it sets a flag to
-		* permanently mark that window as invisible. It works nicely
-		* when vdesk does all the magic itself.
-		*/
-		if (e->atom == vdesk_desktop) fix_visibility_hack(c->window);
-#endif
 	}
 }
 
 void handle_enter_event(XCrossingEvent *e) {
+	Client *c;
 #ifdef VWM
 	int wdesk;
 #endif
-	Client *c = find_client(e->window);
-	if (c) {
+
+	current_screen = find_screen(e->root);
+	/* if (e->window == root && !current) {
+		XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
+		return;
+	} */
+	if ((c = find_client(e->window))) {
 #ifdef VWM
-		wdesk = RD_DESK(c);
-		if (wdesk != GET_VDESK && wdesk != STICKY)
+		wdesk = c->vdesk;
+		if (wdesk != vdesk && wdesk != STICKY)
 			return;
 #endif
 #ifdef COLOURMAP
 		XInstallColormap(dpy, c->cmap);
 #endif
-		if (current && c!=current) {
-			XSetWindowBackground(dpy, current->parent, bg.pixel);
-			XClearWindow(dpy, current->parent);
-		}
-#ifdef VWM
-		XSetWindowBackground(dpy, c->parent, (wdesk==STICKY)?fc.pixel:fg.pixel);
-#else
-		XSetWindowBackground(dpy, c->parent, fg.pixel);
-#endif
-		XClearWindow(dpy, c->parent);
-		current = c;
+		client_update_current(current, c);
+		client_update_current(c, current);
 		XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
 #ifdef MOUSE
-		XGrabButton(dpy, AnyButton, Mod1Mask, c->parent, False,
-			ButtonMask, GrabModeAsync, GrabModeSync, None, None);
-		XGrabButton(dpy, AnyButton, Mod3Mask|Mod1Mask, c->parent, False,
-			ButtonMask, GrabModeAsync, GrabModeSync, None, None);
+		grab_button(c->parent, Mod1Mask, AnyButton);
 #endif
 	}
 }
 
+void handle_leave_event(XCrossingEvent *e) {
+	/* if (e->window == root && !e->same_screen) {
+		client_update_current(current, NULL);
+	} */
+}
+
 #ifdef SHAPE
 void handle_shape_event(XShapeEvent *e) {
 	Client *c = find_client(e->window);
