@@ -150,16 +150,19 @@ extern Atom		xa_wm_delete;
 extern Atom		xa_wm_cmapwins;
 extern Cursor		move_curs;
 extern Cursor		resize_curs;
-extern char		*opt_display;
-extern char		*opt_font;
-extern char		*opt_fg;
-extern char		*opt_bg;
-extern char		**opt_term;
+extern const char	*opt_display;
+extern const char	*opt_font;
+extern const char	*opt_fg;
+extern const char	*opt_bg;
+extern const char	**opt_term;
 extern int		opt_bw;
 #ifdef VWM
-extern char		*opt_fc;
+extern const char	*opt_fc;
 extern int		vdesk;
 #endif
+#ifdef SNAP
+extern int		opt_snap;
+#endif
 #ifdef SHAPE
 extern int		have_shape, shape_event;
 #endif
@@ -202,22 +205,22 @@ void handle_shape_event(XShapeEvent *e);
 /* main.c */
 
 int main(int argc, char *argv[]);
-void scan_windows();
-void setup_display();
+void scan_windows(void);
+void setup_display(void);
 
 /* misc.c */
 #ifdef SANITY
-void sanity_check();
+void sanity_check(void);
 #endif
 #ifdef VDESK_BOTH
 void spawn_vdesk(int todesk, Client *c);
 #endif
 
-/* void do_event_loop(); */
-int handle_xerror(Display *dpy, XErrorEvent *e);
-int ignore_xerror(Display *dpy, XErrorEvent *e);
-void dump_clients();
-void spawn(char *cmd[]);
+/* void do_event_loop(void); */
+int handle_xerror(Display *dsply, XErrorEvent *e);
+int ignore_xerror(Display *dsply, XErrorEvent *e);
+void dump_clients(void);
+void spawn(const char *const cmd[]);
 void handle_signal(int signo);
 #ifdef DEBUG
 void show_event(XEvent e);
