@@ -13,16 +13,6 @@
 #endif
 
 Display		*dpy;
-/*
-int		screen;
-Window		root;
-GC		invert_gc;
-#ifdef VWM
-XColor		fg, bg, fc;
-#else
-XColor		fg, bg;
-#endif
-*/
 int 		num_screens;
 ScreenInfo	*screens;
 ScreenInfo	*current_screen;
@@ -37,16 +27,19 @@ Atom		xa_wm_delete;
 Atom		xa_wm_cmapwins;
 Cursor		move_curs;
 Cursor		resize_curs;
-char		*opt_display = "";
-char		*opt_font = DEF_FONT;
-char		*opt_fg = DEF_FG;
-char		*opt_bg = DEF_BG;
-char		**opt_term = NULL;
+const char	*opt_display = "";
+const char	*opt_font = DEF_FONT;
+const char	*opt_fg = DEF_FG;
+const char	*opt_bg = DEF_BG;
+const char	**opt_term = NULL;
 int		opt_bw = DEF_BW;
 #ifdef VWM
-char		*opt_fc = DEF_FC;
+const char	*opt_fc = DEF_FC;
 int		vdesk = KEY_TO_VDESK(XK_1);
 #endif
+#ifdef SNAP
+int		opt_snap = 0;
+#endif
 #ifdef SHAPE
 int		have_shape, shape_event;
 #endif
@@ -56,6 +49,19 @@ Atom		mwm_hints;
 #endif
 unsigned int numlockmask = 0;
 
+static void *xmalloc(size_t size);
+
+static void *xmalloc(size_t size) {
+	void *ptr = malloc(size);
+	if (!ptr) {
+#ifdef STDIO
+		fprintf(stderr,"out of memory, looking for %d bytes\n",size);
+#endif
+		exit(1);
+	}
+	return ptr;
+}
+
 int main(int argc, char *argv[]) {
 	struct sigaction act;
 	int i;
@@ -78,10 +84,12 @@ int main(int argc, char *argv[]) {
 		else if (!strcmp(argv[i], "-bw") && i+1<argc)
 			opt_bw = atoi(argv[++i]);
 		else if (!strcmp(argv[i], "-term") && i+1<argc) {
-			opt_term = (char **)malloc(3 * sizeof(char *));
+			opt_term = (const char **)xmalloc(3 * sizeof(const char *));
 			opt_term[0] = argv[++i];
 			opt_term[1] = opt_term[0];
 			opt_term[2] = NULL;
+		} else if (!strcmp(argv[i], "-snap") && i+1<argc) {
+			opt_snap = atoi(argv[++i]);
 #ifdef STDIO
 		} else if (!strcmp(argv[i], "-V")) {
 			printf("evilwm version " VERSION "\n");
@@ -90,13 +98,13 @@ int main(int argc, char *argv[]) {
 		} else {
 #ifdef STDIO
 			printf("usage: evilwm [-display display] [-term termprog] [-fg foreground]\n");
-			printf("\t[-bg background] [-bw borderwidth] [-V]\n");
+			printf("\t[-bg background] [-bw borderwidth] [-snap num] [-V]\n");
 #endif
 			exit(2);
 		}
 	}
 	if (!opt_term) {
-		opt_term = (char **)malloc(3 * sizeof(char *));
+		opt_term = (const char **)xmalloc(3 * sizeof(const char *));
 		opt_term[0] = DEF_TERM;
 		opt_term[1] = opt_term[0];
 		opt_term[2] = NULL;
@@ -104,15 +112,10 @@ int main(int argc, char *argv[]) {
 
 	act.sa_handler = handle_signal;
 	sigemptyset(&act.sa_mask);
-/* #ifdef SA_NOCLDSTOP */
-/*	act.sa_flags = SA_NOCLDSTOP; */ /* don't care about STOP, CONT */
-/* #else */
 	act.sa_flags = 0;
-/* #endif */
 	sigaction(SIGTERM, &act, NULL);
 	sigaction(SIGINT, &act, NULL);
 	sigaction(SIGHUP, &act, NULL);
-	/* sigaction(SIGCHLD, &act, NULL); */ /* handling this differently */
 
 	setup_display();
 
@@ -150,8 +153,6 @@ int main(int argc, char *argv[]) {
 				handle_property_change(&ev.xproperty); break;
 			case UnmapNotify:
 				handle_unmap_event(&ev.xunmap); break;
-			/* case Expose:
-				handle_expose_event(&ev.xexpose); break; */
 			default:
 #ifdef SHAPE
 				if (have_shape && ev.type == shape_event) {
@@ -224,7 +225,7 @@ void setup_display() {
 			}
 		}
 	}
-	XFree(modmap);
+	XFreeModifiermap(modmap);
 
 	/* set up GC parameters - same for each screen */
 	gv.function = GXinvert;
@@ -244,14 +245,14 @@ void setup_display() {
 
 	/* now set up each screen in turn */
 	num_screens = ScreenCount(dpy);
-	screens = (ScreenInfo *)malloc(num_screens * sizeof(ScreenInfo));
+	screens = (ScreenInfo *)xmalloc(num_screens * sizeof(ScreenInfo));
 	for (i = 0; i < num_screens; i++) {
 		char *ds, *colon, *dot;
 		ds = DisplayString(dpy);
 		/* set up DISPLAY environment variable to use */
 		colon = rindex(ds, ':');
 		if (colon && num_screens > 1) {
-			screens[i].display = (char *)malloc(14 + strlen(ds));
+			screens[i].display = (char *)xmalloc(14 + strlen(ds));
 			strcpy(screens[i].display, "DISPLAY=");
 			strcat(screens[i].display, ds);
 			colon = rindex(screens[i].display, ':');
