@@ -12,7 +12,7 @@
 #include <stdio.h>
 
 /* Now do this by fork()ing twice so we don't have to worry about SIGCHLDs */
-void spawn(char *cmd[]) {
+void spawn(const char *const cmd[]) {
 	pid_t pid;
 
 	if (current_screen && current_screen->display)
@@ -20,7 +20,8 @@ void spawn(char *cmd[]) {
 	if (!(pid = fork())) {
 		setsid();
 		switch (fork()) {
-			case 0: execvp(cmd[0], &cmd[1]);
+			/* explicitly hack around broken SUS execvp prototype */
+			case 0: execvp(cmd[0], (char *const *)&cmd[1]);
 			default: _exit(0);
 		}
 	}
@@ -66,7 +67,7 @@ void handle_signal(int signo) {
 	exit(0);
 }
 
-int handle_xerror(Display *dpy, XErrorEvent *e) {
+int handle_xerror(Display *dsply, XErrorEvent *e) {
 	Client *c = find_client(e->resourceid);
 
 	/* If this error actually occurred while setting up the new
@@ -91,7 +92,7 @@ int handle_xerror(Display *dpy, XErrorEvent *e) {
 #ifdef XDEBUG
 	fprintf(stderr, "XError %x ", e->error_code);
 #endif
-	/* Kludge aroung IE misbehaviour */
+	/* Kludge around IE misbehaviour */
 	if (e->error_code == 0x8 && e->request_code == 0x0c && e->minor_code == 0x00) {
 #ifdef DEBUG
 		fprintf(stderr, "\thandle_xerror() : IE kludge - ignoring XError\n");
@@ -108,7 +109,7 @@ int handle_xerror(Display *dpy, XErrorEvent *e) {
 	return 0;
 }
 
-int ignore_xerror(Display *dpy, XErrorEvent *e) {
+int ignore_xerror(Display *dsply, XErrorEvent *e) {
 #ifdef DEBUG
 	fprintf(stderr, "ignore_xerror() caught an XErrorEvent: %d\n", e->error_code);
 #endif
