@@ -20,6 +20,9 @@ void make_new_client(Window w, ScreenInfo *s) {
 	long dummy;
 	XWMHints *hints;
 	char *name;
+	/*
+	XClassHint *class;
+	*/
 #ifdef MWM_HINTS
 	PropMwmHints *mhints;
 #endif
@@ -27,6 +30,13 @@ void make_new_client(Window w, ScreenInfo *s) {
 	XGrabServer(dpy);
 
 	c = (Client *)malloc(sizeof(Client));
+	/* Don't crash the window manager, just fail the operation. */
+	if (!c) {
+#ifdef STDIO
+		fprintf(stderr, "out of memory in new_client; limping onward\n");
+#endif
+		return;
+	}
 	/* We do this first of all as a test to see if the window actually
 	 * still exists by the time we've got to create a client structure
 	 * for it (sometimes they vanish too quickly or something, and lots
@@ -49,6 +59,17 @@ void make_new_client(Window w, ScreenInfo *s) {
 	c->next = head_client;
 	head_client = c;
 
+	/*
+	class = XAllocClassHint();
+	if (class) {
+		XGetClassHint(dpy, w, class);
+		fprintf(stderr, "res_name = %s, res_class = %s\n", class->res_name, class->res_class);
+		XFree(class->res_name);
+		XFree(class->res_class);
+		XFree(class);
+	}
+	*/
+
 	/* initialise(c, w); */
 	c->screen = s;
 	c->window = w;
@@ -168,10 +189,12 @@ void init_position(Client *c) {
 	int xmax = DisplayWidth(dpy, c->screen->screen);
 	int ymax = DisplayHeight(dpy, c->screen->screen);
 
-	if (c->size->flags & (/*PSize | */USSize)) {
+	/*
+	if (c->size->flags & (/+PSize | +/USSize)) {
 		c->width = c->size->width;
 		c->height = c->size->height;
 	}
+	*/
 
 	if (c->width < MINSIZE) c->width = MINSIZE;
 	if (c->height < MINSIZE) c->height = MINSIZE;
@@ -236,12 +259,12 @@ void reparent(Client *c) {
 static PropMwmHints *get_mwm_hints(Window w) {
 	Atom real_type; int real_format;
 	unsigned long items_read, bytes_left;
-	PropMwmHints *data;
+	unsigned char *data;
 	if (XGetWindowProperty(dpy, w, mwm_hints, 0L, 20L, False,
 		mwm_hints, &real_type, &real_format, &items_read, &bytes_left,
-		(unsigned char **) &data) == Success
+		&data) == Success
 		&& items_read >= PROP_MOTIF_WM_HINTS_ELEMENTS) {
-		return data;
+		return (PropMwmHints *)data;
 	} else {
 		return NULL;
 	}
