@@ -211,6 +211,50 @@ void show_info(Client *c, KeySym key) {
 }
 
 #ifdef MOUSE
+#ifdef SNAP
+static int absmin(int a, int b) {
+	if (abs(a) < abs(b))
+		return a;
+	return b;
+}
+
+static void snap_client(Client *c) {
+	int dx, dy;
+	Client *ci;
+
+	/* snap to screen border */
+	if (abs(c->x - c->border) < opt_snap) c->x = c->border;
+	if (abs(c->y - c->border) < opt_snap) c->y = c->border;
+	if (abs(c->x + c->width + c->border - DisplayWidth(dpy, c->screen->screen)) < opt_snap)
+		c->x = DisplayWidth(dpy, c->screen->screen) - c->width - c->border;
+	if (abs(c->y + c->height + c->border - DisplayHeight(dpy, c->screen->screen)) < opt_snap)
+		c->y = DisplayHeight(dpy, c->screen->screen) - c->height - c->border;
+
+	/* snap to other windows */
+	dx = dy = opt_snap;
+	for (ci = head_client; ci; ci = ci->next) {
+		if (ci != c && (ci->vdesk == vdesk || ci->vdesk == STICKY)) {
+			if (ci->y - ci->border - c->border - c->height - c->y <= opt_snap && c->y - c->border - ci->border - ci->height - ci->y <= opt_snap) {
+				dx = absmin(dx, ci->x + ci->width - c->x + c->border + ci->border);
+				dx = absmin(dx, ci->x + ci->width - c->x - c->width);
+				dx = absmin(dx, ci->x - c->x - c->width - c->border - ci->border);
+				dx = absmin(dx, ci->x - c->x);
+			}
+			if (ci->x - ci->border - c->border - c->width - c->x <= opt_snap && c->x - c->border - ci->border - ci->width - ci->x <= opt_snap) {
+				dy = absmin(dy, ci->y + ci->height - c->y + c->border + ci->border);
+				dy = absmin(dy, ci->y + ci->height - c->y - c->height);
+				dy = absmin(dy, ci->y - c->y - c->height - c->border - ci->border);
+				dy = absmin(dy, ci->y - c->y);
+			}
+		}
+	}
+	if (abs(dx) < opt_snap)
+		c->x += dx;
+	if (abs(dy) < opt_snap)
+		c->y += dy;
+}
+#endif /* def SNAP */
+
 void drag(Client *c) {
 	XEvent ev;
 	int x1, y1;
@@ -240,6 +284,11 @@ void drag(Client *c) {
 #endif
 				c->x = old_cx + (ev.xmotion.x - x1);
 				c->y = old_cy + (ev.xmotion.y - y1);
+#ifdef SNAP
+				if (opt_snap)
+					snap_client(c);
+#endif
+
 				if ((abs(c->x - old_cx)>5) || (abs(c->y - old_cy)>5))
 					reallymove = 1;
 #ifdef INFOBANNER_MOVERESIZE
@@ -341,7 +390,7 @@ void maximise_vert(Client *c) {
 #ifdef HAS_HIDE
 void hide(Client *c) {
 	if (c) {
-		c->ignore_unmap = 2;
+		c->ignore_unmap += 2;
 #ifdef XDEBUG
 		fprintf(stderr, "screen:XUnmapWindow(parent); ");
 #endif
@@ -356,7 +405,6 @@ void hide(Client *c) {
 #endif
 
 void unhide(Client *c, int raise) {
-	c->ignore_unmap = 0;
 	XMapWindow(dpy, c->window);
 	raise ? XMapRaised(dpy, c->parent) : XMapWindow(dpy, c->parent);
 	set_wm_state(c, NormalState);
@@ -366,7 +414,17 @@ void next(void) {
 	Client *newc = current;
 
 	if (!newc) {
+#ifdef DEBUG
+		fprintf(stderr,"NEXT: no current window, looking on this desktop\n");
+#endif
 		newc = head_client;
+#ifdef VWM
+		if(newc->vdesk != vdesk && newc->vdesk != STICKY) {
+			do {
+				newc = newc->next;
+			} while (newc && newc->vdesk != vdesk && newc->vdesk != STICKY);
+		}
+#endif
 	} else {
 #ifdef VWM
 		do {
@@ -383,12 +441,19 @@ void next(void) {
 		if (newc->vdesk == vdesk || newc->vdesk == STICKY) {
 #endif
 			unhide(newc, RAISE);
+			setmouse(newc->window, 0, 0);
 			setmouse(newc->window, newc->width + newc->border - 1,
 				newc->height + newc->border - 1);
 #ifdef VWM
 		}
 #endif
 	}
+#ifdef DEBUG
+       else {
+               fprintf(stderr,"NEXT: hmm, no next window\n");
+       }
+#endif
+
 }
 
 #ifdef VWM
